##################### FILE ABSTRACT ############################################
##
## Synopsis::         Source for Raw Data routines
##
## File Name::        %M%
## Version::          %I%
## Type::             source
## Date Completed::   %G%
##
## Environment::
##     Language::     Splus
##     Computer::     Compaq/Alpha
##     OS::           DigitalUNIX
##     OSVersion::    4.0
##
## Module Information::
##     Owner::        MDACC
##     Department::   Biostatistics
##     Researcher::   Kim-Anh Do
##     Application::  geneclust
##
###############################################################################

GCRawData.func<-function(nclst,percentToShave,nperm,gapTolerance,fname,
                         percentSupervision,classifile)
{
    cat("PK_MONITOR_READ_MATRIX\n");

    ## Get proper filename (full path)
    if (is.null(fname) == TRUE || nchar(fname) == 0) {
        stop("No data filename specified\n");
    }
    dataFileName<-GetAbsPathName.func(fname);

    ## Determine the type of file we are dealing with:

    ## find file's extension (must be one of the standard ones)
    ext<-substring(dataFileName,
                   first=nchar(dataFileName)-3,
                   last=nchar(dataFileName));

    flagTSV<-as.logical(charmatch(".tsv",ext,nomatch=-1) == 1);
    flagXLS<-as.logical(charmatch(".xls",ext,nomatch=-1) == 1);

    if (flagTSV == FALSE && flagXLS == FALSE) {
       stop("Unsupported data file type!\n");
    }

    colHeaders<-FALSE;
    rowHeaders<-FALSE;
    hasRowHeaders<-tsvFileFormatTest.C(dataFileName,
                                       colHeaders,
                                       rowHeaders)[[3]];
    hasColHeaders<-tsvFileFormatTest.C(dataFileName,
                                       colHeaders,
                                       rowHeaders)[[2]];

    # Store col and row labels of the table into the appropriate files
    outfileCol<-paste(GCoutput,"ColLbls.txt",sep="/");
    outfileRow<-paste(GCoutput,"RowLbls.txt",sep="/");

    ## Check if file has both col and row labels
#    myresult<-ProcessData.func(dataFileName,
#                               outfileCol,
#                               outfileRow,
##                               hasColHeaders,
 ##                              hasRowHeaders,
 #                              classfile,
 ##                              nclst,
  #                             percentToShave,
  #                             nperm,
  #                             gapTolerance,
  #                             percentSupervision);

    ## Display results
 #   DisplayGraphs.func(myresult$C,
 #                      myresult$N,
 #                      myresult$p,
 #                      myresult$nclstAct,
 #                      -1,
 #                      0);

    return();
}


##****************************************************************************
ProcessData.func<-function(dataFileName,
                           outfileCol,
                           outfileRow,
                           hasColHeaders,
                           hasRowHeaders,
                           classfile,
                           nclst,
                           percentToShave,
                           nperm,
                           gapTolerance,
                           percentSupervision)
{
    # Read data from file
    if (hasColHeaders == FALSE) {
        A<-read.table(file=dataFileName,sep="\t",header=FALSE,
                      as.is=TRUE,na.strings="NA",skip=0);
    }
    else {
        A<-read.table(file=dataFileName,sep="\t",header=TRUE,
                      as.is=TRUE,na.strings="NA",skip=0);
    }

    # Create matrix from input data
    B<-na.omit(A);
    N<-dim(B)[1];
    p<-dim(B)[2];
    C<-matrix(0,nrow=N,ncol=p);

    for (i in 1:p) {
        C[,i]<-cbind(B[,names(B)[i]]);
    }

    # No headers are present
    if ((hasRowHeaders == FALSE) && (hasColHeaders == FALSE)) {
        newColLabels<-NULL;
        newRowLabels<-NULL;
    }

    # Column headers are present, no Row headers
    else if ((hasRowHeaders == FALSE) && (hasColHeaders == TRUE)) {
        newColLabels<-names(B);
        newRowLabels<-NULL;
    }

    # Row headers are present, no Column headers
    else if ((hasRowHeaders == TRUE) && (hasColHeaders == FALSE)) {
        newColLabels<-NULL;
        newRowLabels<-row.names(B);
    }

    # Row and Col headers present
    else if ((hasRowHeaders == TRUE) && (hasColHeaders == TRUE)) {
        newColLabels<-names(B);
        newRowLabels<-row.names(B);
    }

    # Validate
    if ((hasColHeaders == TRUE) && (length(newColLabels) != p)) {
        stop("Number of Column labels does not match number of columns in the table\n");
    }
    if ((hasRowHeaders == TRUE) && (length(newRowLabels) != N)) {
        stop("Number of Row labels does not match number of rows in the table\n");
    }

    # Validate
 #   nclstAct<-GenerateLabels.func(newColLabels,
 #                                 newRowLabels,
 #                                 outfileCol,
 #                                 outfileRow,
 #                                 C,
 #                                 N,
 #                                 p,
 #                                 classfile,
 #                                 nclst,
 #                                 percentToShave,
 #                                 nperm,
 #                                 gapTolerance,
 #                                 percentSupervision);

 #   return(C,N,p,nclstAct);
return();
}


#************************************************************************
CallGeneShave.func<-function(C,N,p,classfile,nclst,percentToShave,nperm,
                             gapTolerance,generateLabels,percentSupervision)
{
    W<-as.vector(matrix(t(C),N*p,1));
    is.vector(W);

    verbose<-TRUE;
    if (is.null(classfile) == FALSE) {
        classFileName<-GetAbsPathName.func(classfile);
    }
    else {
        classFileName<-NULL;
    }
    nclstFound<-32767 ## Value is arbitrary, probably better to use 0
    nclstAct<-GeneShave.C(nclst,
                          percentToShave,
                          nperm,
                          gapTolerance,
                          W,
                          N,
                          p,
                          verbose = TRUE,
                          generateLabels,
                          writeMatrix = FALSE,
                          percentSupervision,
                          classFileName,
                          nclstFound)[[13]];

    return(nclstAct);
}


#***************************************************************************
GenerateLabels.func<-function(newColLabels,newRowLabels,outfileCol,outfileRow,
                              C,N,p,classfile,nclst,percentToShave,
                              nperm,gapTolerance,percentSupervision)
{
    # Label processing
    if (is.null(newColLabels) == TRUE) {
        newColLabels<-paste("Sample",c(1:p),sep="");
    }
    write(newColLabels,file=outfileCol);

    if (is.null(newRowLabels) == TRUE) {
        newRowLabels<-paste("GENE",c(1:N),sep="");
    }
    write(newRowLabels,file=outfileRow);
    generateLabels<-FALSE;

    # Perform geneshaving
    nclstAct<-CallGeneShave.func(C,
                                 N,
                                 p,
                                 classfile,
                                 nclst,
                                 percentToShave,
                                 nperm,
                                 gapTolerance,
                                 generateLabels,
                                 percentSupervision);

    return(nclstAct);
}

